//	PCI Interface PLD for v2 pcb
//  For PCI_v2 -- USE WITH 20MHZ OSCILLATOR
//  Author              Greg Burley
//  Organization        OCIW
//  Revision Level      2.1
//  Target Device       EPM7128SQC100-7

// Module Declaration
module pci16 (DQ, D, CLK, RESET, ERXD, ERXFS, ERXCK, ETXD, ETXFS, ETXCK,
	PTWR, PTADR, PTNUM, DXFR, EF, ADDINT, TXBUSY, RXBUSY, WEN, WCLK, 
	REN, RCLK, LD, OE, ERST, FRST, TX_LD, RX_LD,
	G16, G17, G18, G19, G20, G21, G22, G23);

inout	[15:0] DQ;					// PCI data bus -- was DQ[23..0]
output	[15:0] D;					// FIFO data bus
input	CLK, RESET;					// on-board clock and reset
input	ERXD, ERXFS, ERXCK;			// incoming serial data
output	ETXD, ETXFS, ETXCK;			// outgoing serial data
input	PTWR, PTADR; 				// Add-on bus write + address
input	[1:0] PTNUM;				// Add-on bus region select
input	DXFR;						// Add-on data transfer strobe
input	EF;							// FIFO empty flag
output	ADDINT; 					// Add-on Interrupt
output	TXBUSY, RXBUSY;				// serial shiftreg busy flags
output	WEN, WCLK, REN, RCLK;		// fifo write and read signals
output	LD, OE;						// fifo enables
output	ERST, FRST;					// reset signals
output	TX_LD;						// TSHIFT, TCNT load pulse
output  RX_LD;						// RCNT load pulse
output	G16,G17,G18,G19,G20,G21,G22,G23;
// output  [4:0] TCNT,
// output  [4:0] RCNT;

wire [4:0] TCNT, RCNT, TCNT_eq, RCNT_eq;
wire [15:0] TSHIFT, RSHIFT, FCNT;
wire OEa, DRXFS, D1WEN, D2WEN;
wire FCNT_CNT_EN, FCNT_CLK_EN, FCNT_UPDOWN, FCNT_ACLR;

// DEFINE CLOCK EQUIVALENTS & CLOCK CIRCUITS
assign ETXCK = CLK;
assign RCLK = ~CLK;
assign WCLK = ERXCK;

// DEFINE BI-DIRECTIONAL DQ BUS TO S5920 CHIP
// A = (COND) ? TRUE: FALSE;
assign DQ = (OEa) ? FCNT : 16'bz;
assign OEa = (~PTWR & ~DXFR & (PTNUM[1:0] == 'b01) );

// TX SHIFT REGISTER AND SHIFT REGISTER LOAD SIGNAL
lpm_shiftreg tshift(.clock(CLK), .q(TSHIFT), .data(DQ[15:0]), 
	.load(TX_LD), .shiftin(0), .shiftout(ETXD), .aclr(~RESET));
	defparam tshift.lpm_width=16;
	defparam tshift.lpm_direction="RIGHT";

assign TX_LD = (PTWR & ~DXFR & (PTNUM[1:0]=='b00));
assign TXBUSY = TX_LD | ~TCNT_eq[0];

// GENERATE TX FRAME_SYNC SIGNAL
lpm_counter tcnt(.clock(CLK), .q(TCNT), .eq(TCNT_eq), .sset(TX_LD), 
	.cnt_en(~TCNT_eq[0]), .aclr(~RESET));
	defparam tcnt.lpm_width=5;
	defparam tcnt.lpm_direction="DOWN";
	defparam tcnt.lpm_svalue=16;

assign ETXFS = ~TCNT_eq[0];

// GENERATE WRITE ENABLE (FROM RX SHIFT REG TO FIFO)
lpm_counter	rcnt(.clock(~ERXCK), .q(RCNT), .eq(RCNT_eq),
	.sset(RX_LD), .cnt_en(~RCNT_eq[0]));
	defparam rcnt.lpm_width=5;
	defparam rcnt.lpm_direction="DOWN";
	defparam rcnt.lpm_svalue=16;

assign RX_LD = (ERXFS & RCNT_eq[0]);
assign WEN = ~RCNT_eq[1];

dffe u1 (.clk(ERXCK), .d(ERXFS), .q(DRXFS));
assign RXBUSY = (ERXFS | DRXFS);

// RX SHIFT REGISTERS
lpm_shiftreg rshift(.clock(~ERXCK), .shiftin(ERXD), .q(RSHIFT),
 	.enable(ERXFS), .aclr(~RESET));
	defparam rshift.lpm_width=16;
	defparam rshift.lpm_direction="RIGHT";

assign D[15:0] = RSHIFT[15:0];

// GENERATE READ_ENABLE AND OUTPUT_ENABLE SIGNALS FOR FIFO
// OUTPUT_ENABLE SIGNAL FOR FIFO, INTERRUPT FLAG
assign REN = ~(~PTWR & ~DXFR & (PTNUM[1:0]=='b00));
assign OE =  ~(~PTWR & ~DXFR & (PTNUM[1:0]=='b00));
assign ADDINT = ~EF;

// SYNCHRONIZE /WEN FROM WCLK TO RCLK 
dffe u2 (.clk(WCLK), .d(1), .q(D1WEN), .ena(~WEN), .clrn(~D2WEN));
dffe u3 (.clk(RCLK), .d(D1WEN), .q(D2WEN));

// FIFO COUNTER -- INCREMENT ON SYNCHED /WEN, DECREMENT ON /REN
lpm_counter	fcnt(.clock(RCLK), .q(FCNT), .clk_en(FCNT_CLK_EN),
 	.cnt_en(FCNT_CNT_EN), .updown(FCNT_UPDOWN), .aclr(FCNT_ACLR));
	defparam fcnt.lpm_width=16;

assign FCNT_CNT_EN = (~REN ^ D2WEN);
assign FCNT_CLK_EN = (~REN | D2WEN);
assign FCNT_UPDOWN = D2WEN;
assign FCNT_ACLR = ~RESET;

// RESET SIGNALS & DUMMY SIGNALS
assign ERST = RESET;
assign FRST = RESET;
assign LD = 1'b1;

assign G16 = 1'bz;
assign G17 = 1'bz;
assign G18 = 1'bz;
assign G19 = 1'bz;
assign G20 = 1'bz;
assign G21 = 1'bz;
assign G22 = 1'bz;
assign G23 = 1'bz;

endmodule
